// for external usage with bearer token use false.
// inside joomla use true
// const isInsideJoomla = false;

// the internal api route is /api/graphql/
// the external api route is ...thinkiq.net/graphql
const apiRoute = isInsideJoomla ? '/api/graphql/' : authenticator.graphQlEndpoint;

makeRequestAsync = async (query) => {
    let settings = { method: 'POST', headers: {} };
    if (isInsideJoomla) {
        // inside joomla we use form data
        let formData = new FormData();
        formData.append('query', query);
        settings.body = formData;
    } else {
        // externally we use json data
        settings.headers['Content-Type'] = 'application/json';
        settings.body = JSON.stringify({ query: query });
    }

    if (!isInsideJoomla) {
        // externally we need to bring a bearer token
        // use helper js function to obtain bearer token with authenticator
        settings.headers.Authorization = `Bearer ` + await getTokenAsync();
    }

    // make call to obtain graphql response
    let fetchQueryResponse = await fetch(apiRoute, settings);
    return await fetchQueryResponse.json();
}

getRawHistoryDataWithSamplingAsync = async (ids, startDate, endDate, maxSamples = 0) => {
    // this is your graphql query
    let query = `
                    query MyQuery {
                        getRawHistoryDataWithSampling(
                            ids: ${JSON.stringify(ids)}
                            startTime: "${startDate.toISOString()}"
                            endTime: "${endDate.toISOString()}"
                            maxSamples: ${maxSamples}
                        ) {
                            id
                            ts
                            floatvalue
                            intvalue
                            stringvalue
                        }
                    }
                `;

    let queryResponse = await makeRequestAsync(query);

    return queryResponse.data.getRawHistoryDataWithSampling;
}

getTimeSeriesAverages = (data) => {

    // return object where properties are average records per attribute / tag id
    let averages = {};

    // append tsDateTime and sort samples
    data.forEach(tsRecord => {
        tsRecord.tsDateTime = new Date(tsRecord.ts);
    });

    // better save than sorry, sort data by datetime - just in case
    data = data.sort((a, b) => (a.tsDateTime > b.tsDateTime) ? 1 : -1);

    // itterate through all samples
    for (let index = 0; index < data.length; index++) {

        const tsRecord = data[index];
        tsRecord.numvalue = tsRecord.intvalue == null ? tsRecord.floatvalue : tsRecord.intvalue;

        if (!averages[tsRecord.id]) {
            averages[tsRecord.id] = {
                ticks: 0,
                samples: 0,
                valueAverageByTime: 0,
                valueAverageByCount: 0,
                valueMin: tsRecord.numvalue,
                valueMax: tsRecord.numvalue,
                firstRecord: tsRecord,
                lastRecord: null
            };
        }

        const average = averages[tsRecord.id];

        // update min and max
        if (tsRecord.numvalue > average.valueMax) {
            average.valueMax = tsRecord.numvalue;
        }
        if (tsRecord.numvalue < average.valueMin) {
            average.valueMin = tsRecord.numvalue;
        }

        // get sample based average
        average.valueAverageByCount = (average.valueAverageByCount * average.samples + tsRecord.numvalue * 1) / (average.samples + 1);

        // get time based average
        if (average.lastRecord != null) {
            let tsDelta = tsRecord.tsDateTime - average.lastRecord.tsDateTime;
            average.valueAverageByTime = (average.valueAverageByTime * average.ticks + tsRecord.numvalue * tsDelta) / (average.ticks + tsDelta);
            average.ticks += tsDelta;
        }

        average.lastRecord = tsRecord;
        average.samples++;
    }

    return averages;
}

data2traces = (data, ignoreNullValues) => {
    // return object where properties are average records per attribute / tag id
    let traces = {};

    // append tsDateTime and sort samples
    data.forEach(tsRecord => {
        tsRecord.tsDateTime = new Date(tsRecord.ts);
        tsRecord.tsDateTimeISO = tsRecord.tsDateTime.toISOString();
    });

    // better save than sorry, sort data by datetime - just in case
    data = data.sort((a, b) => (a.tsDateTime > b.tsDateTime) ? 1 : -1);

    // itterate through all samples
    for (let index = 0; index < data.length; index++) {

        const tsRecord = data[index];
        tsRecord.numvalue = tsRecord.intvalue == null ? tsRecord.floatvalue : tsRecord.intvalue;

        if (isNaN(tsRecord.numvalue)){

        } else {

            if (!traces[tsRecord.id]) {
                traces[tsRecord.id] = {
                    ticks: 0,
                    samples: 0,
                    firstRecord: tsRecord,
                    lastRecord: null,
                    timestamps: [],
                    timestampsISO: [],
                    values: []
                };
            }

            const trace = traces[tsRecord.id];

            if (tsRecord.numvalue != null || (tsRecord.numvalue == null && !ignoreNullValues)) {
                trace.timestamps.push(tsRecord.tsDateTime);
                trace.timestampsISO.push(tsRecord.tsDateTimeISO);
                trace.values.push(tsRecord.numvalue);
            }

            // update time 
            if (trace.lastRecord != null) {
                let tsDelta = tsRecord.tsDateTime - trace.lastRecord.tsDateTime;
                trace.ticks += tsDelta;
            }

            trace.lastRecord = tsRecord;
            trace.samples++;
        }
    }

    return traces;
}
